;I2C communication routines for PIC16F1829 / MSSP1
;SDA1 = RB4
;SCL1 = RB6
;communication status LED (green/red) connected to RA0 (green) / RA1 (red)


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;I2C Interface initialization
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
i2c_init	nop
		banksel	TRISB
		bsf	TRISB,D'004'		;SDA
		bsf	TRISB,D'006'		;SCL

		banksel	SSP1CON1
;set up I2C BUS for communication as a master for MSSP1
		movlw	B'00101000'		;see Datasheet Page 260
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines - the right ones - testing the SSPxIF bit
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;I2C Communication Routine #1 - Start Condition
;-----------------------------------------------------------------------------
i2c_start	bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		bsf	SSP1CON2,SEN
		goto	i2c_wait_fin
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;I2C Communication Routine #2 - Stop Condition
;-----------------------------------------------------------------------------
i2c_stop	bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		bsf	SSP1CON2,PEN
		call	i2c_wait_fin
		clrf	BSR
		return
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;I2C Communication Routine #3 - Send Byte stored in W-Register
;-----------------------------------------------------------------------------
i2c_send	bcf	PIR1,SSP1IF
		banksel	SSP1BUF
		movwf	SSP1BUF
		clrf	BSR
wait2		btfss	PIR1,SSP1IF
		goto	wait2
		bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		btfss	SSP1CON2,ACKSTAT
		goto	ack_received
		goto	ack_not_rec
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ack_received	clrf	BSR
		goto	ccc2
;-----------------------------------------------------------------------------
ack_not_rec	clrf	BSR
		goto	ccc2
;-----------------------------------------------------------------------------
ccc2		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;I2C Communication Routine #4 - Receive Byte and store to W-Register
;-----------------------------------------------------------------------------
i2c_receive	bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		bsf	SSP1CON2,RCEN
		clrf	BSR
		call	i2c_wait_fin
		banksel	SSP1BUF
		movf	SSP1BUF,0
		clrf	BSR
		return
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;I2C Communication Routine #5 - Send ACK
;-----------------------------------------------------------------------------
i2c_ack		bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		bcf	SSP1CON2,ACKDT			;ACK
		bsf	SSP1CON2,ACKEN
		goto	i2c_wait_fin
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;I2C Communication Routine #6 - Send NOT ACK
;-----------------------------------------------------------------------------
i2c_not_ack	bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		bsf	SSP1CON2,ACKDT			;NOT ACK
		bsf	SSP1CON2,ACKEN
		goto	i2c_wait_fin
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;I2C Communication Routine #7 - Restart Condition
;-----------------------------------------------------------------------------
i2c_restart	bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		bsf	SSP1CON2,RSEN
		goto	i2c_wait_fin
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;I2C Communication -> wait for communication step to finish
;-----------------------------------------------------------------------------
i2c_wait_fin	clrf	BSR
		movlw	D'255'
		movwf	TIMER2

i2c_w_f		nop
		decfsz	TIMER2,1
		goto	cont22
		goto	i2c_time_out
cont22		btfss	PIR1,SSP1IF
		goto	i2c_w_f
		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;I2C communication error - time out occured during I2C I/O operations
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
i2c_time_out	clrf	PORTC
		call	dr4
		call	dr4
		reset
;----------------------------------------------------------------------------
